#include <stdlib.h>
#include <string.h>
#include "train.h"
#include "readField.h"

Boolean readTrainFromTxt(FILE *f, Train *train)
{
	Boolean ok;
	int code;
	TrainStopList stopList;

	ok = fscanf(f, "%d;", &code) == 1;
	if (!ok)
		return false;
	train->code = code;

	ok = readField(train->name, ';', f) == ';';
	if (!ok)
		return false;

	stopList = readTrainStopListFromTxt(f);
	if (stopList == NULL)
		return false;
	train->stopList = stopList;

	return true;	
}

Train* readTrainsFromTxt(FILE *f, int *dim)
{
	Train temp;
	Train *result;
	int count = 0, i = 0;
	while (readTrainFromTxt(f, &temp))
		count++;

	fseek(f, 0, SEEK_SET);
	result = (Train*)malloc(sizeof(Train) * count);
	while (readTrainFromTxt(f, &temp))
	{
		result[i] = temp;
		i++;
	}
	*dim = count;
	return result;
}

void trainToString(Train train, char buffer[])
{
	char trainStopListString[1024];
	trainStopListToString(train.stopList, trainStopListString);
	sprintf(buffer, "Code: %d - Name: %s\n%s\n\n", train.code, train.name, trainStopListString);
}

void printTrains(Train trains[], int dim)
{
	int i;
	char trainString[2048];
	for (i = 0; i < dim; i++)
	{
		trainToString(trains[i], trainString);
		printf(trainString);
	}
}

TrainStopList extractStopListFromTrain(Train train, StationName departureStation, StationName arrivalStation)
{
	return extractStopList(train.stopList, departureStation, arrivalStation);
}

Time tripTime(TrainStopList stopList)
{
	TrainStop departureStop;
	TrainStop arrivalStop;
	departureStop = head(stopList);
	arrivalStop = last(stopList);
	return difference(*(arrivalStop.arrival), *(departureStop.departure));
}

double tripPrice(TrainStopList stopList)
{
	int totalMinutes, priceInterval;
	Time trip = tripTime(stopList);
	totalMinutes = trip.minute + trip.hour * 60;
	priceInterval = totalMinutes / 10 + (totalMinutes % 10 > 0 ? 1 : 0);
	return priceInterval * 0.4;
}

