#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "boolean.h"
#include "trainstoplist.h"

//*
TrainStop last(TrainStopList l)
{
	if (empty(l))
	{
		//Per evitare warning...
		TrainStop fake;
		fake.station[0] = '\0';
		abort();
		return fake;
	}
	else
	{
		TrainStopList listTail = tail(l);
		if (empty(listTail))
		{
			return head(l);
		}
		else
		{
			return last(listTail);
		}
	}
}


TrainStopList copyFromTo(TrainStopList from, TrainStopList to)
{ 
	if (from == to) 
	{
		return cons(head(from), emptyList());
	}
	else 
	{
		return cons(head(from), copyFromTo(tail(from), to));
	}
}

TrainStopList addToList(TrainStop stop, TrainStopList l)
{
	if (empty(l))
	{
		return cons(stop, l);
	}
	else
	{
		TrainStopList temp = l;
		while (!empty(temp->next))
		{
			temp = tail(temp);
		}
		temp->next = cons(stop, emptyList());
		return l;
	}
}

TrainStopList readTrainStopListFromTxt(FILE *f)
{
	TrainStopList list = emptyList();
	TrainStop stop;
	while (readTrainStopFromTxt(f, &stop))
	{
		list = addToList(stop, list);
	}
	return list;
}


//*
TrainStopList findNextStop(TrainStopList stopList, StationName name)
{
	while (stopList != NULL && !inStation(stopList->data, name))
	{
		stopList = stopList->next;
	}
	return stopList;
}

//*
TrainStopList extractStopList(TrainStopList stopList, StationName departureStation, StationName arrivalStation)
{
	TrainStopList departureStop = NULL;
	TrainStopList arrivalStop = NULL;

	departureStop = findNextStop(stopList, departureStation);
	if (departureStop != NULL)
	{
		arrivalStop = findNextStop(departureStop->next, arrivalStation);
		if (arrivalStop != NULL)
		{
			return copyFromTo(departureStop, arrivalStop);
		}
	}
	return NULL;
}

//*
void trainStopListToString(TrainStopList stopList, char buffer[])
{
	char trainStopString[256];
	while (stopList != NULL)
	{
		trainStopToString(stopList->data, trainStopString);
		sprintf(buffer, "\t%s\n", trainStopString);

		buffer += strlen(buffer);
		stopList = stopList->next;
	}
}

/* OPERAZIONI PRIMITIVE */
TrainStopList emptyList(void)		/* costruttore lista vuota */
{ 
	return NULL; 
}

Boolean empty(TrainStopList l)	/* verifica se lista vuota */
{ 
	return l == NULL;
}

TrainStopList cons(TrainStop e, TrainStopList l)
{ 
	TrainStopList t;       /* costruttore che aggiunge in testa alla lista */
	t = (TrainStopList)malloc(sizeof(TrainStopListElement));
	t->data = e;
	t->next = l;
	return t;
}

TrainStop head(TrainStopList l) /* selettore testa lista */
{ 
	if (empty(l)) 
	{
		//Per evitare warning...
		TrainStop fake;
		fake.station[0] = '\0';
		abort();
		return fake;
	}
	else
	{
		return l->data; 
	}
}

TrainStopList tail(TrainStopList l)         /* selettore coda lista */
{ 
	if (empty(l))
	{
		abort();
		return NULL;
	}
	else 
	{
		return l->next; 
	}
}



/* OPERAZIONI NON PRIMITIVE */


/*  ATTENZIONE:  la showlist() e' definita solo per element di tipo int  */
/* VERSIONE ITERATIVA: */
/*
void   showlist(TrainStopList l)
{ 
  printf("[");
  while (!empty(l)) 
          { printf("%d",head(l));
	      l=tail(l);
	      if (!empty(l)) printf(", "); 
          }
  printf("]\n");
}
*/


/* Lunghezza di una lista (ricorsiva) */
/*
int	length(TrainStopList l)
{ if (empty(l)) return(0);
  else return 1+length(tail(l));
}
*/


/* Lunghezza di una lista (iterativa) */
/*
int	length_it(TrainStopList l)
{ int n=0;
  while (!empty(l))
	 { n++; 
          l=tail(l); }
  return(n);
}
*/


/* appartenenenza di un elemento - ricorsiva  */
/*
boolean member(element el, TrainStopList l)
{ if (empty(l)) return 0;
  else { if (el==head(l)) return 1;
	  else return member(el,tail(l)); }
}
*/


/* appartenenenza di un elemento - iterativa  */
/*
boolean member_it(element el,TrainStopList l)
{ while (!empty(l))
	  { if (el==head(l)) return 1;
           else  l= tail(l); }
  return 0;
}
*/


/* crea una copia della lista l */
/*
TrainStopList copy (TrainStopList l)
{ if (empty(l)) return l;
  else return cons(head(l),copy(tail(l)));
}
*/


/*
TrainStopList delete(element el, TrainStopList l)
{ if (empty(l)) return emptylist();
  else {if (el==head(l)) return tail(l);
	 else return cons(head(l),delete(el,tail(l))); }
}
*/


 /* inversione di una lista - ricorsiva */
/*
TrainStopList reverse (TrainStopList l)
{
  if (empty(l)) return emptylist();
  else return append(reverse(tail(l)),
			cons(head(l),emptylist()));
}
*/


/* funzione ausiliaria */
/*
TrainStopList rev(TrainStopList l2, TrainStopList l1)
{
 if  (empty(l1)) return l2;
 else return rev(cons(head(l1),l2),tail(l1)); 
}
*/


/* inversione di una lista - tail ricorsiva      */
/*
TrainStopList reverse_tail(TrainStopList l)
{
  return rev(emptylist(),l);
}
*/


/*  ATTENZIONE:  la insord() e' definita solo per element di tipo int*/
/*
TrainStopList insord(element el, TrainStopList l) {
  if (empty(l)) return cons(el,l);
  else if (el <= head(l)) return cons(el,l);
  else return cons(head(l), insord(el,tail(l)));
}
*/